#ifndef __X3DEXPORTER_H
#define __X3DEXPORTER_H

#include <string>
using std::string;

#include <iostream>
#include <fstream>

#include "Exporter.h"
#include "Containers.h"

class SceneGraphNode;
class Mesh;
class Intersectable;
class MeshInstance;
class Polygon3;
class ViewCell;
class BspTree;

class X3dExporter : public Exporter
{
  std::ofstream stream;

public:

  X3dExporter(const string filename);

  ~X3dExporter();


  bool
  ExportRays(const vector<Ray> &rays,
	     const float length=1000,
	     const RgbColor &color = RgbColor(1,1,1));

  bool
  ExportKdTree(const KdTree &tree);
  
  bool ExportBspTree(const BspTree &tree);

  bool
  ExportScene(SceneGraphNode *root) 
  {
    ExportSceneNode(root);
    return true;
  }
  
  virtual void 
  ExportPolygon(Polygon3 *poly);

  virtual void 
  ExportPolygons(const PolygonContainer &polys);

  virtual bool
  ExportBox(const AxisAlignedBox3 &box);

  virtual void
  ExportMeshInstance(MeshInstance *mi);

  virtual void
  ExportIntersectable(Intersectable *object);

  virtual void
  ExportMesh(Mesh *mesh);

  virtual void 
  ExportViewCell(ViewCell *viewCell);

  virtual void 
  ExportViewCells(const ViewCellContainer &viewCells);

  virtual void 
  ExportBspSplitPlanes(const BspTree &tree);
  
  virtual void 
  ExportBspSplits(const BspTree &tree);

  virtual void
  ExportLeavesGeometry(const BspTree &tree, const vector<BspLeaf *> &leaves);

  bool
  ExportRays(const RayContainer &rays,
			 const float length=1000,
			 const RgbColor &color = RgbColor(1,1,1));

  virtual void
  ExportBspViewCellPartition(const BspTree &tree, const int maxPvs = 0);

  virtual void 
  ExportBspLeaves(const BspTree &tree);

protected:

  virtual void
  ExportSceneNode(SceneGraphNode *node);

  bool
  ExportKdTreeRayDensity(const KdTree &tree);

  bool
  ExportBspTreeRayDensity(const BspTree &tree);  
};
 


#endif
